      .486
      .model flat, stdcall
      option casemap :none

      include \masm32\include\windows.inc
      include \masm32\include\user32.inc
      include \masm32\include\kernel32.inc
      include \masm32\include\masm32.inc
      include \masm32\macros\macros.asm
      includelib \masm32\lib\masm32.lib
      includelib \masm32\lib\user32.lib
      includelib \masm32\lib\kernel32.lib

      ; Define a data type for the drive parameters returned by
      ; the Interrupt 13h Extensions GetDriveParameters function.
      ; The bufferSize element must be set to the size of the
      ; structure or the function will fail.
      ;
      ; Note that for fixed disk drives that exceed the normal
      ; BIOS limit of 8.4GB (8,455,200,768 bytes), the cylinders,
      ; heads, and sectors per track parameters will be standard
      ; values that correspond to the normal BIOS limit, not to
      ; the actual values for the drive (this convention allows
      ; the normal BIOS functions error free access to the first
      ; 8.4GB of the drive). The total sectors and bytes per
      ; sector values will be correct for the drive.
      ;
      ; The IBM/MS Interrupt 13h Extensions, BTW, use Logical
      ; Block Addressing (LBA) rather than the Cylinder, Head,
      ; Sector (CHS) addressing that the normal BIOS functions
      ; use.
      ;
      Int13ExtDriveParameters STRUCT
         bufferSize        dw SIZEOF Int13ExtDriveParameters
         informationFlags  dw ?
         cylinders         dd ?
         heads             dd ?
         sectorsPerTrack   dd ?
         totalSectorsLow   dd ?
         totalSectorsHigh  dd ?
         bytesPerSector    dw ?
      Int13ExtDriveParameters ENDS

      ; Constants for the bit fields of the
      ; informationFlags element.
      DmaBoundaryErrorsHandledTransparently EQU 1
      ChsInformationValid EQU 2
      RemovableDrive EQU 4
      WriteWithVerifySupported EQU 8
      DriveHasChangeLine EQU 16
      DriveCanBeLocked EQU 32
      ChsInformationSetToMaxSupported EQU 64

.data
      ; This will be the name of the DOS program.
      comFileName db "GET13XDP.COM",0

      hFile       dd 0
      bRead       dd 0
      buffer      db 30 dup(0)
.code
start:
      ; Create the DOS program and run it.
      ; No forward references with INVOKE.
      mov   eax,OFFSET comStart
      mov   ecx,OFFSET comEOF - OFFSET comStart
      invoke write_disk_file,ADDR comFileName,eax,ecx
      test  eax,eax
      jnz   @F
      print SADD(13,10,"write_disk_file (GET13XDP.COM) failed")
   @@:
      invoke shell,ADDR comFileName
      REPEAT 4
         ; Make up to 4 attempts to open the data file,
         ; waiting 200ms between attempts.
         mov   eax,OFFSET dataFileName
         invoke CreateFile,eax,GENERIC_READ,
            FILE_SHARE_READ,NULL,OPEN_EXISTING,NULL,NULL
         cmp   eax,INVALID_HANDLE_VALUE
         jne   @F
         print SADD(13,10,"CreateFile (open GET13XDP.DAT) failed")
         invoke Sleep,200
      ENDM
   @@:
      ; Read parameters into structure var and close the file.
      mov   hFile,eax
      mov   eax,OFFSET parameters
      mov   ecx,SIZEOF parameters
      invoke ReadFile,hFile,eax,ecx,ADDR bRead,NULL
      invoke CloseHandle,hFile

      ; Display selected parameters.
      mov   eax,parameters.cylinders
      invoke dw2a,eax,ADDR buffer
      print SADD(13,10,"cylinders = ")
      print ADDR buffer

      mov   eax,parameters.heads
      invoke dw2a,eax,ADDR buffer
      print SADD(13,10,"heads = ")
      print ADDR buffer

      mov   eax,parameters.sectorsPerTrack
      invoke dw2a,eax,ADDR buffer
      print SADD(13,10,"sectorsPerTrack = ")
      print ADDR buffer

      mov   eax,parameters.totalSectorsLow
      invoke dw2a,eax,ADDR buffer
      print SADD(13,10,"totalSectorsLow = ")
      print ADDR buffer

      mov   eax,parameters.totalSectorsHigh
      invoke dw2a,eax,ADDR buffer
      print SADD(13,10,"totalSectorsHigh = ")
      print ADDR buffer

      movzx eax,parameters.bytesPerSector
      invoke dw2a,eax,ADDR buffer
      print SADD(13,10,"bytesPerSector = ")
      print ADDR buffer

      print SADD(13,10,"Press enter to exit...",13,10)
      invoke StdIn,addr buffer,1

      ; Delete the DOS program and the data file.
      invoke DeleteFile,ADDR comFileName
      test   eax,eax
      jnz   @F
      print SADD(13,10,"DeleteFile (GET13XDP.COM) failed")
   @@:
      mov   eax,OFFSET dataFileName
      invoke DeleteFile,eax
      test   eax,eax
      jnz   @F
      print SADD(13,10,"DeleteFile (GET13XDP.DAT) failed")
   @@:
      exit

; End the 32-bit segment created by the .MODEL directive.
_TEXT ENDS

; Start a 16-bit segment.
_TEXT16 SEGMENT USE16

comStart:
      ; Start of the DOS program that will actually call the
      ; BIOS function. The program will terminate on the first
      ; error.

      ; Jump around data.
      jmp   @F

      parameters     Int13ExtDriveParameters <>
      dataFileName   db "GET13XDP.DAT",0

      ; Windows will load the program into memory starting at
      ; offset 100h. Calculate the run-time offset addresses.
      OFFSET_PARAMETERS EQU 100h + OFFSET parameters - OFFSET comStart
      OFFSET_DATAFILENAME EQU 100h + OFFSET dataFileName - OFFSET comStart
   @@:
      ; Make sure the Interrupt 13h Extensions are installed,
      ; and that the major version is at least 1.0, and that
      ; bit 0 of the API subset support bitmap is set,
      ; indicating that the fixed disk access subset (func
      ; 42h-44h, 47h, and 48h) is supported.
      mov   ax,4100h
      mov   bx,55aah
      mov   dx,80h      ; First fixed disk.
      int   13h

      ; Carry set = function not supported.
      ; Note that under Windows XP and probably 2000 the
      ; function will always return with carry set.
      jc    comEnd

      ; If the value returned in BX is not AA55h (note
      ; reversal), then the extensions are not installed.
      cmp   bx,0aa55h
      jne   comEnd

      ; The major version is returned in AH.
      cmp   ah,1
      jb    comEnd

      ; The API subset support bitmap is returned in CX.
      test  cl,1
      jz    comEnd

      ; Call the Interrupt 13h Extensions Get Drive Parameters
      ; function for the first fixed disk.
      mov   ax,4800h
      mov   dx,80h
      mov   si,OFFSET_PARAMETERS
      int   13h
      jc    comEnd

      ; Create the data file and write the parameters to it.
      ATTR_NORMAL EQU 0
      mov   dx,OFFSET_DATAFILENAME
      mov   cx,ATTR_NORMAL
      mov   ah,3ch
      int   21h
      jc   comEnd
      mov   bx,ax    ; Handle now in AX.
      mov   cx,SIZEOF parameters
      mov   dx,OFFSET_PARAMETERS
      mov   ah,40h
      int   21h

      ; Handle will be closed and file buffers flushed
      ; on terminate.
   comEnd:
      mov   ax,4c00h
      int   21h

comEOF:

end start
